'use strict';
const CONST = require("../../constants.js");
const LOG   = require("../../core/Logger.js");

const BagBasedHandlerBase = require("./BagBasedHandlerBase.js");
const HandlerException = require("./HandlerException.js");

const AlertDataModel = require("../dal/AlertDataModel.js");

class DataHandler extends BagBasedHandlerBase {
	static getRoute() {
		return "/:bagId/alert";
	}

	async _processGET() {
		let alertSettings = this._bag.getAlertData();
		if (!alertSettings) alertSettings = new AlertDataModel();
		return this.respondOk({
			data : alertSettings.getData()
		});
	}

	async _processPUT() {
		let alertSettings = new AlertDataModel(this._body);
		let valid = alertSettings.validate();

		if (valid !== true) return this.respondBadRequest(valid);

		this._bag.setAlertData(alertSettings);

		return this._saveBagData()
					.then(r => this.respondOk({
						data : alertSettings.getData()
					}));
	}

	async _processPOST() {
		return this.respondMethodNotAllowed("POST");
	}

	async _processDELETE() {
		return this.respondMethodNotAllowed("DELETE");
	}

	_handleError(error) {
		if (error instanceof HandlerException) return this.respondBadRequest(error.message);
		LOG.error("Error during data update", error);
		return this.oops();
	}

	needsMasterAccess() {
		return false;
	}
}

module.exports = DataHandler;