'use strict';
const CONST = require("../../constants.js");
const LOG   = require("../../core/Logger.js");

const APIRequestHandlerBase = require("../APIRequestHandlerBase.js");
const HandlerException = require("./HandlerException.js");

class BagBasedHandlerBase extends APIRequestHandlerBase {
	async _init() {
		this._authKey = this._headers[CONST.API_REQUEST_AUTH_HEADER];
		if (this._body && this._headers["content-type"] === "application/json") {
			try {
				this._body = JSON.parse(this._body);
			} catch (e) {
				LOG.info("Invalid JSON received", this._body, e);
				return Promise.reject(this.respondBadRequest("Invalid JSON payload"));
			}
		}

		if (this._body && typeof(this._body) !== "object") {
			LOG.info("Payload is not JSON, or header('" + this._headers["content-type"] + "') is missing", this._body);
			return Promise.reject(this.respondBadRequest("Invalid request payload, should be JSON and content-type should be provided as well."));
		}
	}

	async _setup() {
		return this._dal.getBagData(this._vars.bagId)
			.then(res => {
				if (!res) return Promise.reject(this.respondNotFound("Unknown bag"));
				this._bag = res;
			});
	}

	async _isAuthenticated() {
		return this._authKey ? true : false;
	}

	async _hasAccess() {
		LOG.debug(this._logPrefix, "Handler requires master acces:", this.needsMasterAccess());
		return this.needsMasterAccess() ?
			this._bag.hasMasterAccess(this._authKey) :
			this._bag.hasClientAccess(this._authKey);
	}

	needsMasterAccess() {
		return true;
	}

	_saveBagData() {
		return this._dal.updateBagData(this._bag);
	}
}

module.exports = BagBasedHandlerBase;