'use strict';
const CONST = require("../../constants.js");
const LOG   = require("../../core/Logger.js");

const BagBasedHandlerBase = require("./BagBasedHandlerBase.js");
const HandlerException = require("./HandlerException.js");

class DataPositionHandler extends BagBasedHandlerBase {
	static getRoute() {
		return "/:bagId/data/position";
	}

	async _processGET() {
		let lastData = this._bag.getLastData();
		if (!lastData) return this.respondNotFound("No last data found");
		let data = lastData.getData();
		return this.respondOk({
			data : {
				ts : data.ts,
				lon : data.lon,
				lat : data.lat
			}
		});
	}

	async _processPUT() {
		return this.respondMethodNotAllowed("PUT");
	}

	async _processPOST() {
		return this.respondMethodNotAllowed("POST");
	}

	async _processDELETE() {
		return this.respondMethodNotAllowed("DELETE");
	}

	needsMasterAccess() {
		return false;
	}
}

module.exports = DataPositionHandler;