'use strict';
const CONST = require("../../constants.js");
const LOG   = require("../../core/Logger.js");

const BagBasedHandlerBase = require("./BagBasedHandlerBase.js");
const HandlerException = require("./HandlerException.js");

class TokenResetHandler extends BagBasedHandlerBase {
	static getRoute() {
		return "/:bagId/token/reset";
	}

	async _processGET() {
		return this.respondMethodNotAllowed("GET");
	}

	async _processPUT() {
		let data = this._body.data || {};
		let token = data.token;
		let type  = data.type || "client";

		if (type !== "client") {
			return this.respondBadRequest("Token type must be client!");
		}

		if (!token || typeof token !== "string" || token.length == 0) {
			return this.respondBadRequest("Invalid token!");
		}

		this._bag.updateClientKey(token);

		return this._saveBagData()
					.then(r => this.respondOk());
	}

	async _processPOST() {
		return this.respondMethodNotAllowed("PUT");
	}

	async _processDELETE() {
		return this.respondMethodNotAllowed("DELETE");
	}

	_handleError(error) {
		if (error instanceof HandlerException) return this.respondBadRequest(error.message);
		LOG.error("Error during data update", error);
		return this.oops();
	}

	needsMasterAccess() {
		return true;
	}
}

module.exports = TokenResetHandler;