'use strict';
const HASH_LENGTH = 2;

const PATH_BAG = "/bag/";
const EXTENSION_BAG_DATA = ".json";

class BlobStorage {
	constructor(connector, storageRootPath, dirHashDepth) {
		this._connector = connector;
		this._root      = storageRootPath;
		this._hashDepth = dirHashDepth;
	}

	_dirHash(fileName) {
		let clean = fileName.replace(/[^a-zA-Z0-9]/g,"").padStart(this._hashDepth * HASH_LENGTH, "0");
		let parts = [];
		for (let i = 0; i < this._hashDepth - 1; i++) {
			parts[i] = clean.substr(i * HASH_LENGTH, HASH_LENGTH);
		}

		parts[this._hashDepth - 1] = clean.substr((this._hashDepth - 1) * HASH_LENGTH);

		return parts.join("/");
	}

	_getBagPath(bagId) {
		return this._root + PATH_BAG + this._dirHash(bagId) + "/";
	}

	getBagLogs(bagId) {
		let path = this._getBagPath(bagId);
		return this._connector.readBlob(path, bagId + EXTENSION_BAG_DATA);
	}

	saveBagLogs(bagId, data, ts) {
		let path = this._getBagPath(bagId);
		let name = bagId + "_" + (ts || 0);
		return this._connector.createPath(path)
			.then(r => this._connector.writeBlob(path, name + EXTENSION_BAG_DATA, data));
	}

	appendBagLogs(bagId, data, ts) {
		let path = this._getBagPath(bagId);
		let name = bagId + "_" + (ts || 0);
		return this._connector.createPath(path)
			.then(r => this._connector.appendBlob(path, name + EXTENSION_BAG_DATA, data));
	}
}

module.exports = BlobStorage;