'use strict';
const CONST = require("../constants.js");
const LOG   = require("./Logger.js");

const CacheBase = require("./CacheBase.js");

class CacheLRUImpl extends CacheBase {
	constructor(size) {
		super();
		this._maxSize = size;
		this._size    = 0;

		this._items = {};
		this._listOfDeath = {
			head : null,
			tail : null
		};
		LOG.info("LRU (Least Recently Used) cache initialized with a size of", this._maxSize);
	}

	_createRecordPointer(id) {
		return {
			p  : null,
			n  : null,
			id : id
		}
	}

	_addToList(wrapper) {
		let {head, tail} = this._listOfDeath;
		let pnt          = wrapper.r;

		if (tail === null) {
			this._listOfDeath.head = pnt;
		} else {
			tail.n = pnt;
			pnt.p  = tail;
		}

		this._listOfDeath.tail = pnt;
	}

	_removeFromList(wrapper) {
		let {head, tail} = this._listOfDeath;
		let pnt          = wrapper.r;

		if (pnt === head) this._listOfDeath.head = pnt.n;
		if (pnt === tail) this._listOfDeath.tail = pnt.p;

		if (pnt.p) {
			pnt.p.n = pnt.n;
		}

		if (pnt.n) {
			pnt.n.p = pnt.p;
		}

		pnt.p = null;
		pnt.n = null;
	}

	_moveToEnd(wrapper) {
		this._removeFromList(wrapper);
		this._addToList(wrapper);
	}

	has(id) {
		return this._items[id] !== undefined;
	}

	get(id, def) {
		let item = this._items[id];
		if (!item) return def;
		this._moveToEnd(item);
		return item.d;
	}

	add(id, item) {
		if (this._items[id]) {
			this._moveToEnd(this._items[id]);
			this._items[id].d = item;
		} else {
			let wrapper = {
				d : item,
				r : this._createRecordPointer(id)
			}
			this._items[id] = wrapper;

			this._addToList(wrapper);
			this._size++;
		}

		while (this._size > this._maxSize) {
			this.remove(this._listOfDeath.head.id);
		}
	}

	remove(id) {
		let item = this._items[id];
		if (!item) return;
		this._removeFromList(item);
		delete this._items[id];
		this._size--;
	}

	clear() {
		this._items = {};
		this._listOfDeath = {
			head : null,
			tail : null
		};
	}

	each(cb) {
		let pnt = this._listOfDeath.head;
		while (pnt !== null) {
			cb(pnt.id, this._items[pnt.id].d);
			pnt = pnt.n;
		}
	}
}

module.exports = CacheLRUImpl;