'use strict';
const ClusteringBase = require("./ClusteringBase.js");

class DBSCAN extends ClusteringBase {
	buildClusters(minCount, epsilon) {
		let labels = [];
		for (let i = 0; i < this._records.length; i++) {
			labels[i] = 0;
		}

		let clusterIdx = 0;
		for (let i = 0; i < this._records.length; i++) {
			if (labels[i] !== 0) continue;

			let neigh = this.inRange(i, epsilon);

			if (neigh.length < minCount) {
				labels[i] = -1;
				continue;
			}

			clusterIdx++;
			labels[i] = clusterIdx;

			let set     = {};
			while (neigh.length > 0) {
				let current = neigh.shift();
				if (labels[current] === -1) {
					labels[current] = clusterIdx;
				}
				if (labels[current] !== 0) continue;

				labels[current] = clusterIdx;
				let newNeigh = this.inRange(current, epsilon);
				if (newNeigh.length >= minCount) {
					for (let j = 0; j < newNeigh.length; j++) {
						let nn = newNeigh[j];
						if (labels[nn] <= 0 && !set[nn]) {
							neigh.push(nn);
							set[nn]	= true;
						}
					}
					//neigh = neigh.concat(newNeigh);
				}
			}
		}

		return labels;
	}

	inRange(idx, epsilon) {
		let ret = [];
		let ee  = epsilon * epsilon;
		let r1  = this._records[idx];
		for (let i = 0; i < this._records.length; i++) {
			if ((ee >= this.distanceSquare(r1, this._records[i])) && (i !== idx)) {
				ret.push(i);
			}
		}

		return ret;
	}

	distanceSquare(r1, r2) {
		let s = 0;
		for (let i = 0; i < r1.length; i++) {
			s += (r1[i] - r2[i]) * (r1[i] - r2[i]);
		}

		return s;
	}
}

module.exports = DBSCAN;