'use strict';
const HASH_LENGTH = 2;

const PATH_BAG = "/bag/";
const EXTENSION_BAG_DATA = ".json";

class BlobStorage {
	constructor(connector, storageRootPath, dirHashDepth) {
		this._connector = connector;
		this._root      = storageRootPath;
		this._hashDepth = dirHashDepth;
	}

	_dirHash(fileName) {
		let clean = fileName.replace(/[^a-zA-Z0-9]/g,"").padStart(this._hashDepth * HASH_LENGTH, "0");
		let parts = [];
		for (let i = 0; i < this._hashDepth - 1; i++) {
			parts[i] = clean.substr(i * HASH_LENGTH, HASH_LENGTH);
		}

		parts[this._hashDepth - 1] = clean.substr((this._hashDepth - 1) * HASH_LENGTH);

		return parts.join("/");
	}

	_getBagPath(bagId) {
		return this._root + PATH_BAG + this._dirHash(bagId) + "/";
	}

	_readBlobList(path, lst, res) {
		return new Promise((resolve, reject) => {
			if (lst.length > 0) {
				let item = lst.pop();
				this._connector.readBlob(path + "/", item)
					.then(r => {res = res.concat(r)})
					.then(r => resolve(this._readBlobList(path, lst, res)));
			} else {
				resolve(res);
			}
		});
	}

	getBagLogs(blobName) {
		let path = this._getBagPath(blobName);

		return this._connector.list(path).then(r => this._readBlobList(path, r, []));
	}

	saveBagLogs(bagId, data, ts) {
		let path = this._getBagPath(bagId);
		let name = bagId + "_" + (ts || 0);
		return this._connector.createPath(path)
			.then(r => this._connector.writeBlob(path, name + EXTENSION_BAG_DATA, data));
	}

	appendBagLogs(bagId, data, ts) {
		let path = this._getBagPath(bagId);
		let name = bagId + "_" + (ts || 0);
		return this._connector.createPath(path)
			.then(r => this._connector.appendBlob(path, name + EXTENSION_BAG_DATA, data));
	}
}

module.exports = BlobStorage;