module.exports = class Mutex {

	constructor () {
		 this.queue = []
		 this.working = false
	}

	async _next () {

		// Enforce single thread
		if (this.working) {
			// Another call is still running
			return
		}

		// Mark that we are running jobs from this call exclusively
		this.working = true		

		// There is at least one pending job
		while (this.queue.length > 0) {
			let job = this.queue.shift()

			// Signal the job to start working
			job.begin(job.mutex)

			// Wait for it to finish
			await job.done
		}

		// All done
		this.working = false
	}

	// Request a mutex
	request () {

		let job = {
			begin: null,
			done: null,
			mutex: {
				release: null
			}
		}

		let begin = new Promise((resolve) => {
			job.begin = resolve
		})

		job.done = new Promise((resolve) => {
			job.mutex.release = resolve
		})

		// Add the job to the queue
		this.queue.push(job)

		// Trigger a job start
		this._next()

		// The begin promise will be resolved when the job may start
		return begin
	}
}
