'use strict';
const CONST = require("../constants.js");
const LOG     = require("../core/Logger.js");
const Client  = require("./Client.js");
const Channel = require("./Channel.js");

class ClientStore {
	constructor(config) {
		this._config = config;

		this._clients  = {};
		this._channels = {};
	}

	_getIdFromRequest(req) {
		return req.connection.remoteAddress + ":" + req.connection.remotePort;
	}

	addClient(socket, request) {
		let key = this._getIdFromRequest(request);

		if (this._clients[key]) {
			LOG.error("Possible leak, duplicate client found: " + key);
		}

		this._clients[key] = new Client(key, socket, this._config, this);
	}

	removeClient(id) {
		let client = this._clients[id];
		if (client) {
			delete this._clients[id];
		}
	}

	removeChannel(id) {
		let channel = this._channels[id];
		if (channel) {
			delete this._channels[id];
		}
	}

	provideChannel(callId1, callId2, client) {
		let channelId = Channel.generateChannelId(callId1, callId2);
		if (!this._channels[channelId]) {
			this._channels[channelId] = new Channel(callId1, callId2, this);
		}

		this._channels[channelId].addClient(client);

		return this._channels[channelId];
	}


	closeAll() {
		for (let i in this._clients) {
			this._clients[i].close(1012, "Service is shutting down");
		}
	}
}

module.exports = ClientStore;