'use strict';
const LOG   = require("../core/Logger.js");

const ServiceBase = require("../core/ServiceBase.js");
const WebSocket   = require("ws");
const ClientStore = require("./ClientStore.js");

class TunnelService extends ServiceBase {
	_initialize(done) {
		this._port = this._config.get("tunnel.port", 8080);
		this._clientStore = new ClientStore(this._config);

		LOG.info("Initializing websocket service on port " + this._port);

		this._wss  = new WebSocket.Server({
			port: this._port
		});

		done();
	}

	_run() {
		this._wss.on("connection", (socket, request) => this._connectionHandler(socket, request));
	}

	_connectionHandler(socket, request) {
		LOG.info("Client " + request.connection.remoteAddress + ":" + request.connection.remotePort + " connected.");
		this._clientStore.addClient(socket, request);
	}

	_terminate(done) {
		LOG.info("Closing sockets");
		this._clientStore.closeAll();
		LOG.info("Closing server");
		this._wss.close(() => done());
		LOG.info("Waiting for connection cleanup");
	}
}

module.exports = TunnelService;