'use strict';
const CONST = require("../../constants.js");
const LOG   = require("../../core/Logger.js");

const MESSAGE_TYPE_ACK="ack";
const MESSAGE_TYPE_DATA="data";
const MESSAGE_TYPE_CONTROL="control";

class ServerMessageBuilder {
	constructor(type, channel) {
		this._type    = type;
		this._channel = channel || null;
		this._data    = null;
		this._seq     = null;
		this._ok      = true;

		this._error        = 0;
		this._errorMessage = undefined;
	}

	setChannel(channel) {
		this._channel = channel;
		return this;
	}

	setData(data) {
		this._data = data;
		return this;
	}

	setSequence(seq) {
		this._seq = seq;
		return this;
	}

	setError(code, message) {
		this._error        = code;
		this._errorMessage = message;
		this._ok           = false;
		return this;
	}

	_generateMeta(err, errStr) {
		return {
			v      : CONST.PROTOCOL_VERSION,
			cmpt   : CONST.PROTOCOL_COMPATIBLITY_VERSION,
			chn    : this._channel ? true : false,
			cli    : this._channel ? this._channel.getConnections() : 0,
			err    : err || 0,
			//This will be JSON serialized, so if these parameters are not provided, these will be left out, since there is no JSON encode for undefined
			errStr : errStr
		}
	}

	build() {
		let msg = {
			type : this._type,
			m    : this._generateMeta()
		}

		if (this._type === MESSAGE_TYPE_ACK) {
			msg.seq = this._seq;
			msg.ok  = this.ok;
		}

		if (this._type === MESSAGE_TYPE_DATA) {
			msg.d = this._data;
		}

		return JSON.stringify(msg);
	}
}

module.exports = {
	message : (type, channel) => new ServerMessageBuilder(type, channel),
	MESSAGE_TYPE_ACK     : MESSAGE_TYPE_ACK,
	MESSAGE_TYPE_DATA    : MESSAGE_TYPE_DATA,
	MESSAGE_TYPE_CONTROL : MESSAGE_TYPE_CONTROL
};