'use strict';
const SecUtil               = require("../../core/SecUtil.js");

class UserModel {
	constructor(key, rawData) {
		this._defaultData();
		this._id = key;
		if (rawData) {
			if (rawData._id !== key) {
				throw new Error(
					"Key doesn't match raw data's key '" + key + "' + vs '" + rawData._id + "'"
				)
			}
			this.parseData(rawData);
		}
	}

	parseClientData(rawData) {
		if (rawData.password) {
			this.updatePassword(rawData.password);
		}
		this._bagId = rawData.bagId || this._bagId;
		this._clientToken = rawData.clientToken || this._clientToken;
		this._name  = rawData.name || this._name;
	}


	parseData(rawData) {
		this._password        = rawData.password;
		this._name            = rawData.name;
		this._bagId           = rawData.bagId;
		this._clientToken     = rawData.clientToken;
		this._activated       = rawData.activated;
		this._activationToken = rawData.activationToken;
	}

	getData() {
		return {
			_id             : this._id,
			password        : this._password,
			bagId           : this._bagId,
			clientToken     : this._clientToken,
			activated       : this._activated,
			activationToken : this._activationToken,
			name            : this._name
		}
	}

	getClientData() {
		return {
			bagId       : this._bagId,
			clientToken : this._clientToken,
			name        : this._name,
			email       : this._id
		}
	}

	getId() {
		return this._id;
	}

	getBagId() {
		return this._bagId;
	}

	_defaultData() {
		this._id              = null;
		this._password        = null;
		this._bagId           = null;
		this._clientToken     = null;
		this._activated       = false;
		this._activationToken = null;
		this._name            = "Unknown";
	}

	activate(token) {
		if (this.isActivated()) return true;

		if (this._activationToken === null || this._activationToken !== token) {
			throw new Error("Invalid activation token");
		}

		this._activated = true;
		return true;
	}

	setActivationToken(token) {
		this._activated = false;
		this._activationToken = token;
	}

	isActivated() {
		return this._activated;
	}

	setActivated(activated) {
		this._activated = activated;
	}

	validatePassword(password) {
		if (!this._password) return false;
		return SecUtil.comparePassword(password, this._password);
	}

	updatePassword(rawKey) {
		this._password = SecUtil.generatePasswordString(SecUtil.salt().replace("=", ""), "sha256", rawKey);
	}

}

module.exports = UserModel;