'use strict';

const CONST  = require("../../constants.js");
const LOG    = require("../../core/Logger.js");

const UserBasedHandlerBase = require("./UserBasedHandlerBase.js");
const HandlerException = require("./HandlerException.js");

class AuthHandler extends UserBasedHandlerBase {
	static getRoute() {
		return "/user/:userId/authenticate";
	}

	_getAuth() {
		if (!this._body.email) throw new HandlerException("Invalid auth data received, missing user name");
		if (this._body.email !== this._vars.userId) throw new HandlerException("Inconsistent user name received");

		return {
			id  : this._body.email,
			ttl : this._startTime + CONST.AUTH_TTL_SEC * 1000
		};
	}

	async _isAuthenticated() {
		return true;
	}

	async _hasAccess() {
		return true;
	}

	async _processPUT() {
		if (!this._user.validatePassword(this._body.password)) {
			return this.respondUnauthorized("Invalid username or password!");
		}

		this._refreshAuthCookie();

		return this.respondOk();
	}

	async _setup() {
		return this._dal.getUserData(this._vars.userId)
			.then(res => {
				this._user = res;
			}).catch(e => Promise.reject(this.respondUnauthorized("Invalid username or password!")));
	}
}

module.exports = AuthHandler;