'use strict';
const CONST  = require("../../constants.js");
const LOG    = require("../../core/Logger.js");

const UserBasedHandlerBase = require("./UserBasedHandlerBase.js");
const HandlerException = require("./HandlerException.js");

class UserHandler extends UserBasedHandlerBase {
	static getRoute() {
		return "/user/:userId/reset";
	}

	async _processGET() {
		return this.respondMethodNotAllowed("GET");
	}

	async _processPUT() {
		return this.respondMethodNotAllowed("PUT");
	}

	async _processDELETE() {
		return this.respondMethodNotAllowed("DELETE");
	}

	async _processPOST() {
		if (!this._body || !this._body.email || !this._body.bagId) {
			return this.respondBadRequest("Missing data fields!");
		}

		if (this._user.getBagId() !== this._body.bagId || this._vars.userId !== this._body.email) {
			return this.respondBadRequest("Invalid identity data provided!");
		}

		let newPassword = Math.round(Math.random() * 60466176).toString(36).toUpperCase();
		this._user.updatePassword(newPassword);

		return this._saveUserData()
			.then(r => this._notifyUser(newPassword))
			.then(r => this.respondOk({
				data: this._user.getClientData()
			}))
			.catch(e => this._handleError(e));
	}

	_notifyUser(newPassword) {
		return this._mail.sendMail({
			to : this._user.getId(),
			subject: "New Password for SMB",
			text: "Your new password is: " + newPassword
		});
	}

	_handleError(error) {
		if (error instanceof HandlerException) return this.respondBadRequest(error.message);
		LOG.error("Error during user update", error);
		return this.oops();
	}

	async _isAuthenticated() {
		return true;
	}

	async _hasAccess() {
		return true;
	}
}

module.exports = UserHandler;