'use strict';

const CONST  = require("../../constants.js");
const LOG    = require("../../core/Logger.js");

const UserBasedHandlerBase = require("./UserBasedHandlerBase.js");
const HandlerException = require("./HandlerException.js");

class UserHandler extends UserBasedHandlerBase {
	static getRoute() {
		return "/user/:userId";
	}

	async _processGET() {
		return this.respondOk({
			data: this._user.getClientData()
		});
	}

	async _processPOST() {
		if (!this._body) {
			return this.respondBadRequest("Missing data fields!");
		}

		if (this._body.password && (!this._body.oldPassword || !this._user.validatePassword(this._body.oldPassword))) {
			return this.respondUnauthorized("Invalid old password provided!");
		}

		this._user.parseClientData(this._body);

		return this._saveUserData()
			.then(r => this.respondOk({
				data: this._user.getClientData()
			}))
			.catch(e => this._handleError(e));
	}

	_handleError(error) {
		if (error instanceof HandlerException) return this.respondBadRequest(error.message);
		LOG.error("Error during user update", error);
		return this.oops();
	}
}

module.exports = UserHandler;