'use strict';
const ManagedObjectBase = require("./ManagedObjectBase.js");

const {
	UnimplementedMethodException,
	MissingFieldException
} = require('../Exception/Exceptions.js');

class ScreenBase extends ManagedObjectBase {
	constructor() {
		super()
	}

	/**
	 * This method should return true if the screen can be displayed with the current validity, and false otherwise
	 * Validity is a custom field in the store, to allow screen availability management.
	 *
	 * @args {string} availability - The validity information
	 * @return boolean
	 */
	checkScreenAvailability(availability) {
		return true;
	}

	/**
	 * When the screen validity check fails this method will be invoked to get the name of the fallback screen.
	 * Should this method return falsy (null, "", false... etc) the initial screen will be used
	 *
	 * @args {string} availability - The validity information
	 * @return string
	 */
	getFallbackScreen(availability) {
		return null;
	}

	getScreen() {
		throw new UnimplementedMethodException("Screens must implement getScreen method!");
	}
}

//ScreenBase.prototype.MANAGED_NAME = null;

module.exports = ScreenBase;