'use strict';
const Util         = require("../../Util/Util.js");
const DataPipe     = require("../DataPipe.js");
const RESTProvider = require("../Provider/RESTProvider.js");
const RESTPathSplitter = require("../Middleware/RESTPathSplitter.js");

//TODO: Validation!
class RESTBuilder {
	constructor(baseUrl, middlewares, provider) {
		this._baseUrl = baseUrl;

		this._interfaceData = {};

		this._currentBasePath = "";
		this._currentKey      = "";

		this._middlewares     = middlewares || [];
		this._provider        = provider;
	}

	_addPathData(path, method, operation) {
		let id = this._interfaceData[this._currentKey];

		id[operation] = {
			pp     : Util.createPathPattern(this._baseUrl, this._currentBasePath, path),
			method : method
		};
	}

	connectModel(key, model, path, middlewares, provider) {
		this._currentBasePath = path;
		this._currentKey      = key;
		this._interfaceData[key] = {
			model : model
		};

		return this;
	}

	standardCSL() {
		this.create("", "POST", "create");
		this.save("/:id", "PUT", "save");
		this.load("/:id", "GET", "load");
		return this;
	}

	create(path, method) {
		this._addPathData(path, method, "create");
		return this;
	}

	save(path, method) {
		this._addPathData(path, method, "save");
		return this;
	}

	load(path, method) {
		this._addPathData(path, method, "load");
		return this;
	}

	buildPipes() {
		let ret = {};

		for (let i in this._interfaceData) {
			let pipeDesc = this._interfaceData[i];
			let mw       = new RESTPathSplitter(
				pipeDesc["create"] ? pipeDesc["create"].pp     : undefined,
				pipeDesc["create"] ? pipeDesc["create"].method : undefined,
				pipeDesc["save"] ? pipeDesc["save"].pp     : undefined,
				pipeDesc["save"] ? pipeDesc["save"].method : undefined,
				pipeDesc["load"] ? pipeDesc["load"].pp     : undefined,
				pipeDesc["load"] ? pipeDesc["load"].method : undefined
			);

			ret[i] = {
				pipe  : new DataPipe(this._provider, [mw, ...this._middlewares]),
				model : pipeDesc.model
			};
		}

		return ret;
	}
}

module.exports = RESTBuilder;
