"use strict";
const BusinessLogicBase = require("../../smbfw/src/smbfw.js").BusinessLogicBase;
const SmbfwActions      = require("../../smbfw/src/smbfw.js").Actions;
const DataManagerActions = require("./DataManagerBL.js").Actions;

class LoginScreenBL extends BusinessLogicBase {
	autoLoginHandler(state, dispatch) {
		if (document.cookie) {
			let cookies = document.cookie.split(";");
			let jar     = {};
			cookies.forEach(cookie => {
				let parts = cookie.split("=");
				jar[parts[0].trim()] = parts[1].trim();
			});

			if (jar["smbuiauth"] && jar["smbuiuser"]) {
				dispatch(
					SmbfwActions.updateState({
						"auth.user": jar["smbuiuser"]
					})
				);

				this._fetchUserDataAndProceed(jar["smbuiuser"], dispatch);
			}
		}
	}

	loginHandler(state, dispatch, email, password) {
		let login = this._smbfw.dm.createData("authenticate", null, email);
		login.setAuthParams(email, password);

		dispatch(SmbfwActions.increaseLock("full"));
		login.save().then(token => {
			dispatch(
				SmbfwActions.updateState({
					"auth.user": email
				})
			);
		}).catch(e => {
			dispatch(SmbfwActions.openSnack(this._smbfw.l.localize("error.login"), "Error"));
			this._error(e);
			// ModalWindowBL.errorWindow(dispatch,
			// 	'modal.error_messages.error_title',
			// 	"modal.error_messages.login_error_message")
		}).then(() => this._fetchUserDataAndProceed(email, dispatch))
		.then(() => dispatch(SmbfwActions.decreaseLock("full")));
	}

	_fetchUserDataAndProceed(email, dispatch) {
		return this._smbfw.dm.loadData("user", email).then(user => {
			dispatch(
				SmbfwActions.updateState({
					"dynamic.user": user
				})
			);
			return this._smbfw.dm.loadData("notification_config", null, user.getBagId())
				.catch(e => {
					this._warn(e);
					dispatch(SmbfwActions.openSnack(this._smbfw.l.localize("warning.clientToken"), "Warning"));
				});
		}).then(config => {
			dispatch(
				SmbfwActions.updateState({
					"dynamic.notificationConfig": config
				})
			);
		}).then(res => {
			dispatch(DataManagerActions.actionStartFetch());
			dispatch(SmbfwActions.changeScreen("DashboardScreen"));
		}).catch(e => {
			dispatch(SmbfwActions.openSnack(this._smbfw.l.localize("error.fetchData"), "Error"));
			this._error(e);
		});
	}
}

LoginScreenBL.prototype.MANAGED_NAME = "login_screen_bl";

BusinessLogicBase.generateActions(LoginScreenBL, {
	actionLogin : "loginHandler",
	autoLogin   : "autoLoginHandler"
});

module.exports = LoginScreenBL;
