const React     = require("react");
const connect   = require("../../../smbfw/src/smbfw.js").connect;
const LayoutScreenBase     = require("./LayoutScreenBase.jsx").default;
const TrackingMap     = require("../presentational/TrackingMap.jsx").default;
const DashboardScreenActions = require("../../businessLogic/TrackBagScreenBL.js").Actions;

import GoogleMapReact from 'google-map-react';
import BusinessCenterIcon from '@material-ui/icons/BusinessCenter';
import Card from '@material-ui/core/Card';
import CardActions from '@material-ui/core/CardActions';
import CardContent from '@material-ui/core/CardContent';
import Typography from '@material-ui/core/Typography';
import Grid from '@material-ui/core/Grid';

const MapMarker = require("../presentational/MapMarker.jsx").default;

class DashboardScreen extends LayoutScreenBase {
	constructor(props) {
		super(props);
	}

	_genericStateUpdate(event) {
		this.setState({ [event.target.name]: event.target.value });
	}

	static getDerivedStateFromProps(props, state) {
		let bag = props.bag;
		return {
			lat : bag ? bag.getLatitude() : 0,
			lon : bag ? bag.getLongitude() : 0,
			battery : bag ? bag.getBatteryCharge() : 0
		}
	}

	renderScreen() {
		let $_l = this.props.$_l;
		let bag = this.props.bag;
		let lat = bag ? bag.getLatitude() : 0;
		let lon = bag ? bag.getLongitude() : 0;
		let battery = bag ? bag.getBatteryCharge() : 0;
		let gadgets = bag ? bag.getTotalGadgets() : 0;
		let availableGadgets = bag ? bag.getAvailableGadgets() : 0;

		return (
			<div>
				<Typography variant="h3" component="h2" style={{margin : "20px 0px 20px 0px"}}>
					{$_l("screen.dashboard.title")}
				</Typography>

				<div className="dashboardContainer">
						<div className="mapContainer">
						<Card className="infoCard">
							<CardContent style={{paddingBottom:"16px"}}>
								<TrackingMap
									lon={lon}
									lat={lat}
									width="100%"
									height="75vh"
									text={$_l("screen.track_bag.map_tag")}
								/>
							</CardContent>
						</Card>
					</div>
					<div className="cardContainer">
						<Card className="infoCard">
							<CardContent>
								<Typography gutterBottom variant="h4" component="h2">
									{$_l("screen.dashboard.position")}
								</Typography>
								<Typography gutterBottom variant="h6" component="h2">
									{$_l("screen.dashboard.longitude")}
								</Typography>
								<Typography gutterBottom variant="h5" component="h2" align="right" color="primary">
									{lon}
								</Typography>
								<Typography gutterBottom variant="h6" component="h2">
									{$_l("screen.dashboard.latitude")}
								</Typography>
								<Typography gutterBottom variant="h5" component="h2" align="right" color="primary">
									{lat}
								</Typography>
							</CardContent>
						</Card>
						<Card className="infoCard">
							<CardContent>
								<Typography gutterBottom variant="h4" component="h2">
									{$_l("screen.dashboard.battery")}
								</Typography>
								<Typography gutterBottom variant="h2" component="h2" align="center" style={{lineHeight : 2}}  color="primary">
									{battery}%
								</Typography>
							</CardContent>
						</Card>
						<Card className="infoCard">
							<CardContent>
								<Typography gutterBottom variant="h4" component="h2">
									{$_l("screen.dashboard.gadgets")}
								</Typography>
								<Typography gutterBottom variant="h2" component="h2" align="center" style={{lineHeight : 2}}  color="primary">
									{availableGadgets} / {gadgets}
								</Typography>
							</CardContent>
						</Card>
					</div>
				</div>
			</div>
		);
	}
}

const mapStateToProps = state => ({
	bag : state.dynamic.bag,
	...(LayoutScreenBase.getStateToProps(state))
});

const mapDispatchToProps = DashboardScreenActions.toMapDispatchToProps({
	...(LayoutScreenBase.getDispatchToProps())
});

export default connect(
	mapStateToProps,
	mapDispatchToProps
)(DashboardScreen);
