const webpack = require('webpack');
const HtmlWebPackPlugin = require("html-webpack-plugin");
const MiniCssExtractPlugin = require("mini-css-extract-plugin");
const TerserPlugin = require('terser-webpack-plugin');
const paths = require("path");

module.exports = (env, argv) => {
	const MODE     = (env ||  argv.mode);
	const OUT_PATH = paths.resolve(__dirname, 'dist');

	return {
		entry: {
			index     : "./src/index.js",
			main      : "./src/scss/main.scss"
		},
		output: {
			path: OUT_PATH
		},
		module: {
			rules: [
				{
					test: /\.(js|jsx)$/,
					use: [
						{
							loader: "inspect-loader",
							options: {
								callback(inspect) {
									console.log("JSX: ", arguments[0].context.resourcePath);
								}
							}
						},
						{
							options: {
								configFile: "./babel.config.js"
							},
							loader: "babel-loader"
						}
					]
				},
				{
					test: /\.html$/,
					use: [
						{
							loader: "html-loader"
						}
					]
				},
				{
					test: /\.scss$/,
					use: [
						{
							loader: "inspect-loader",
							options: {
								callback(inspect) {
									console.log("SCSS: ", arguments[0].context.resourcePath);
								}
							}
						},
						{
							loader: MiniCssExtractPlugin.loader,
							options: {
								hmr: argv.mode === "development"
							}
						},
						{
							loader: "css-loader",
							options: { url: false }
						},
						"sass-loader"
					]
				},
				{
					test: /\.json$/,
					exclude: /node_modules/,
					use: [
						{
							loader: "json-loader"
						}
					]
				}
			]
		},
		plugins: [
			new HtmlWebPackPlugin({
				template: "./src/index.html",
				filename: "./index.html",
				excludeChunks : ["constants"]
			}),
			new MiniCssExtractPlugin({
				filename: "[name].css",
				chunkFilename: "[id].css"
			})
		],
		node: {
			fs: "empty",
			net: "empty",
			tls: "empty"
		},
		optimization: {
			minimizer: [
				new TerserPlugin({
					terserOptions: {
						keep_classnames: true,
						keep_fnames: true
					},
				}),
			],
		},
		devServer: {
		    compress: true,
		    disableHostCheck: true
		}
	}
};
