'use strict';
const CONST = require("../constants.js");
const LOG   = require("./Logger.js");

const fs    = require('fs');

class Configuration {
	constructor(path) {
		this._config    = {};
		this._hasConfig = false;

		let configPath = process.env[CONST.ENV_CONFIG_FILE_VAR] || path || CONST.DEFAULT_CONFIG_FILE;
		LOG.info("Config file location from environment: " + (process.env[CONST.ENV_CONFIG_FILE_VAR] || "N/A"));
		LOG.info("Config file location constructor: " + (path || "N/A"));
		LOG.info("Configuration is being loaded from: " + configPath);

		try {
			let json = fs.readFileSync(configPath);
			this._config = this._flatten(JSON.parse(json));
			this._hasConfig = true;
		} catch (e) {
			LOG.info("Warning, error loading configuration: " + e);
		}
	}

	_flatten(obj, parentPath) {
		let ret = {};
		parentPath = parentPath ? (parentPath + ".") : "";

		for (let i in obj) {
			ret[parentPath + i] = obj[i];
			if (typeof obj[i] === "object" && obj !== null) {
				ret = {...ret, ...this._flatten(obj[i], parentPath + i)};
			}
		}

		Object.freeze(ret);

		return ret;
	}

	get(key, def) {
		return this._config[key] || def;
	}

	hasConfig() {
		return this._hasConfig;
	}
}

module.exports = Configuration;